/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

// This is an automatically generated file.

#ifndef YARP_ROSMSG_deprecated1_stereo_msgs_DisparityImage_h
#define YARP_ROSMSG_deprecated1_stereo_msgs_DisparityImage_h

#include <yarp/conf/system.h>

YARP_COMPILER_DEPRECATED_WARNING(<stereo_msgs_DisparityImage.h> header is deprecated. Use <yarp/rosmsg/stereo_msgs/DisparityImage.h> instead)

#include <yarp/rosmsg/stereo_msgs/DisparityImage.h>
#include <std_msgs_Header.h>
#include <sensor_msgs_Image.h>
#include <sensor_msgs_RegionOfInterest.h>
#include <yarp/conf/api.h>

YARP_DEPRECATED_MSG("stereo_msgs_DisparityImage is deprecated, use yarp::rosmsg::stereo_msgs::DisparityImage instead")
typedef yarp::rosmsg::stereo_msgs::DisparityImage stereo_msgs_DisparityImage;

#endif // YARP_ROSMSG_deprecated1_stereo_msgs_DisparityImage_h
