/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

// This is an automatically generated file.

#ifndef YARP_ROSMSG_deprecated1_geometry_msgs_InertiaStamped_h
#define YARP_ROSMSG_deprecated1_geometry_msgs_InertiaStamped_h

#include <yarp/conf/system.h>

YARP_COMPILER_DEPRECATED_WARNING(<geometry_msgs_InertiaStamped.h> header is deprecated. Use <yarp/rosmsg/geometry_msgs/InertiaStamped.h> instead)

#include <yarp/rosmsg/geometry_msgs/InertiaStamped.h>
#include <std_msgs_Header.h>
#include <geometry_msgs_Inertia.h>
#include <yarp/conf/api.h>

YARP_DEPRECATED_MSG("geometry_msgs_InertiaStamped is deprecated, use yarp::rosmsg::geometry_msgs::InertiaStamped instead")
typedef yarp::rosmsg::geometry_msgs::InertiaStamped geometry_msgs_InertiaStamped;

#endif // YARP_ROSMSG_deprecated1_geometry_msgs_InertiaStamped_h
