/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef IFRAMETRANSFORMTEST_H
#define IFRAMETRANSFORMTEST_H

#include <yarp/dev/IFrameTransform.h>

using namespace yarp::dev;
using namespace yarp::sig;
using namespace yarp::os;

void exec_frameTransform_test_1(IFrameTransform* iTf);
void exec_frameTransform_test_2(IFrameTransform* iTf);

#endif
