/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

// This is an automatically generated file.

#ifndef YARP_ROSMSG_deprecated2_tf2_msgs_TF2Error_h
#define YARP_ROSMSG_deprecated2_tf2_msgs_TF2Error_h

#include <yarp/conf/system.h>

YARP_COMPILER_DEPRECATED_WARNING(<tf2_msgs/TF2Error.h> header is deprecated. Use <yarp/rosmsg/tf2_msgs/TF2Error.h> instead)

#include <yarp/rosmsg/tf2_msgs/TF2Error.h>
#include <tf2_msgs_TF2Error.h>
#include <yarp/conf/api.h>

namespace tf2_msgs {

YARP_DEPRECATED typedef yarp::rosmsg::tf2_msgs::TF2Error TF2Error;

} // namespace tf2_msgs

#endif // YARP_ROSMSG_deprecated2_tf2_msgs_TF2Error_h
