/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef IMAP2DTEST_H
#define IMAP2DTEST_H

#include <yarp/dev/IMap2D.h>

using namespace yarp::dev;
using namespace yarp::dev::Nav2D;
using namespace yarp::sig;
using namespace yarp::os;

void exec_iMap2D_test_1(IMap2D* imap);
void exec_iMap2D_test_2(IMap2D* imap);

#endif
