/*
 * SPDX-FileCopyrightText: 2006-2021 Istituto Italiano di Tecnologia (IIT)
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include <yarp/dev/IOdometry2D.h>

yarp::dev::Nav2D::IOdometry2D::~IOdometry2D() = default;
